;----------------------------------------------------------------------------------------------
;
;  Copyright (c) 2009, GERI, Liverpool John Moores University. All rights reserved.
;       Unauthorized reproduction prohibited.
;
;+
;  FILE:
;       Traps.pro
;
;  PURPOSE:
;       This application analyses trap images received from the Blaze Media Pro software.
;       http://www.blazemp.com/
;
;----------------------------------------------------------------------------------------------

;pro trapsFiles, fFiles, sFiles, sTimes
;
;	; Get number of files in the folder
;	fElements = N_ELEMENTS(fFiles)
;	; Array, which contains time of the frame
;	fTimes = dblarr(fElements)
;
;	; Get file name and transform it into the time
;	FOR i=0,fElements-1 DO BEGIN
;
;		fTimes[i] = double( file_basename(fFiles[i],'.jpg'))
;
;	ENDFOR
;
;	; Get sorting index for sorting procedure
;	sIndex = sort(fTimes)
;
;	; Run sorting procedure
;	sTimes = fTimes[sIndex]
;	sFiles = fFiles[sIndex]
;
;end

pro trapDetection, image, xdim, ydim, xpos, ypos, xdelta, ydelta, filterValue, bubbleX, bubbleY

	;delta = 25

	x1 = xpos - xdelta
	if (x1 lt 0) then begin
		x1 = 1
	endif

	y1 = ypos - ydelta
	if (y1 lt 0) then begin
		y1 = 1
	endif

	x2 = xpos + xdelta
	if (x2 ge xdim-11) then begin
		x2 = xdim - 11
	endif

	y2 = ypos + ydelta
	if (y2 ge ydim) then begin
		y2 = ydim - 1
	endif

	r = REFORM(image[0, *, *])

	iSobled = sobel(r)

	for i=x1, x2 do begin
		for j=y1, y2 do begin
			if(iSobled[i,j] gt filterValue)then begin
				iSobled[i,j] = 1
			endif else begin
				iSobled[i,j] = 0
			endelse
		endfor
	endfor

	Sd = REPLICATE(1, 2, 2)
	Se = REPLICATE(1, 1, 1)

	iEroded = erode(dilate(iSobled,Sd),Sd)

	iEroded[x1:x2,y2] = 0

	for i=x1, x2 do begin
		for j=y2, y1,-1 do begin
			if iEroded[i,j] eq 0 then begin
				if iEroded[i-1,j] eq 1 then begin
					if iEroded[i,j+1] eq 1 then begin

						if iEroded[i+1,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+2,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+3,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+4,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+5,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+6,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+7,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+8,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+9,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+1,j] eq 1 then begin
							iEroded[i,j] = 1
						endif


					endif
				endif
			endif

		endfor
	endfor

	firstMax = 0
	secondMax = 0
	distanceMax = 0
	levelMax = 0

	flag = 0

	for j=y1, y2 do begin
		first = 0
		second = 0
		flag = 0

		for i=x1, x2 do begin

			if(iEroded[i,j] eq 1 AND iEroded[i-1,j] eq 0 AND flag eq 0)then begin
				first = i
				flag = 1
			endif

			if(iEroded[i,j] eq 0 AND iEroded[i-1,j] eq 1 AND flag eq 1)then begin
				second = i-1
				flag = 0

				if(distanceMax lt (second-first)) then begin
					firstMax = first
					secondMax = second
					distanceMax = second-first
					levelMax = j
				endif

			endif

		endfor
	endfor

	i = firstMax+0.5*(secondMax-firstMax)
	j = LevelMax
	while iEroded[i,j] ne 0 AND j lt y2 do begin
		j = j+1
	endwhile

	bubbleY1 = j-1

	j = LevelMax
	while iEroded[i,j] ne 0 AND j ge 0 do begin
		j = j-1
	endwhile

	bubbleY2 = j+1

	j = bubbleY2 + 0.5*(bubbleY1-bubbleY2)

	bubbleX = i
	bubbleY = j

	iEroded[i,j] = 0

    tvscl, iEroded

end

pro trapsDetection, image, xdim, ydim, filterValue

	r = REFORM(image[0, *, *])
	g = REFORM(image[1, *, *])
	b = REFORM(image[2, *, *])

	ax = dblarr(xdim,ydim)
	ay = dblarr(xdim,ydim)

	rr = dblarr(xdim,ydim)
	rr[*,*] = r[*,*]

	for i=1, xdim-1 do begin
		for j=0, ydim-1 do begin
			ax[i,j] = abs(rr[i,j] - rr[i-1,j])
		endfor
	endfor

	for j=1, ydim-1 do begin
		for i=0, xdim-1 do begin
			ay[i,j] = abs(rr[i,j] - rr[i,j-1])
		endfor
	endfor
	;tvscl, ay


	iSobled = sobel(rr)

	for i=0, xdim-1 do begin
		for j=0, ydim-1 do begin
			if(iSobled[i,j] gt filterValue)then begin
				iSobled[i,j] = 1
			endif else begin
				iSobled[i,j] = 0
			endelse
		endfor
	endfor

	Sd = REPLICATE(1, 2, 2)
	Se = REPLICATE(1, 1, 1)
	;iEroded = erode(erode(dilate(iSobled,Sd),Sd),Se)
	iEroded = erode(dilate(iSobled,Sd),Sd)
	;iEroded = iSobled

	iEroded[*,ydim-1] = 0

	for i=1, xdim-11 do begin
		for j=ydim-2, 0,-1 do begin
			if iEroded[i,j] eq 0 then begin
				if iEroded[i-1,j] eq 1 then begin
					if iEroded[i,j+1] eq 1 then begin

						if iEroded[i+1,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+2,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+3,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+4,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+5,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+6,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+7,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+8,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+9,j] eq 1 then begin
							iEroded[i,j] = 1
						endif
						if iEroded[i+1,j] eq 1 then begin
							iEroded[i,j] = 1
						endif


					endif
				endif
			endif

		endfor
	endfor


   tvscl, iEroded

end
pro trapsDraw, sState

	;  Draw.
	;
	sState.drawWindowID->Draw, sState.oView

end

; -----------------------------------------------------------------------------
;
;  Purpose:  Event handler
;
pro trapsEvent, $
    sEvent     ; IN: event structure

    ;  Quit the application using the close box.
    ;
    if (TAG_NAMES(sEvent, /STRUCTURE_NAME) EQ $
        'WIDGET_KILL_REQUEST') then begin
        WIDGET_CONTROL, sEvent.top, /DESTROY
        RETURN
    endif

    ;  Get the info structure from top-level base.
    ;
    WIDGET_CONTROL, sEvent.top, GET_UVALUE=sState, /NO_COPY

    ;  Determine which event.
    ;
    WIDGET_CONTROL, sEvent.id, GET_UVALUE=eventval

    ;  Take the following action based on the corresponding event.
    ;
    case eventval of

		'PXTEXT' : begin

			WIDGET_CONTROL, sState.wPixelXField, GET_VALUE=xPixelSizeStr

			xPixelSize = DOUBLE(xPixelSizeStr[0])

			str = string(DOUBLE(sState.sX[sState.sCounter])*xPixelSize/DOUBLE(sState.sMagnification))
			WIDGET_CONTROL, sState.wXrLabel, SET_VALUE=str, /DYNAMIC_RESIZE

		end

		'PYTEXT' : begin

			WIDGET_CONTROL, sState.wPixelYField, GET_VALUE=yPixelSizeStr

			yPixelSize = DOUBLE(yPixelSizeStr[0])

		   	str = string(DOUBLE(sState.sY[sState.sCounter])*yPixelSize/DOUBLE(sState.sMagnification))
			WIDGET_CONTROL, sState.wYrLabel, SET_VALUE=str, /DYNAMIC_RESIZE

		end

        'SLIDER_FILTER' : begin

			WIDGET_CONTROL, sState.fSlider, GET_VALUE=i

			sState.sFilterValue = i

			if(sState.sElements gt 0) then begin

			endif

        end    ;  of SLIDER

		'M_FACTOR' : begin

			listValue = WIDGET_INFO( sEvent.id, /DROPLIST_SELECT)

            case listValue of

                0 : begin
                sState.sMagnification = 2
                end

                1 : begin
                sState.sMagnification = 5
                end

                2 : begin
                sState.sMagnification = 10
                end

                3 : begin
                sState.sMagnification = 20
                end

                4 : begin
                sState.sMagnification = 30
                end
                
                5 : begin
                sState.sMagnification = 40
                end
                
                6 : begin
                sState.sMagnification = 60
                end
                
                7 : begin
                sState.sMagnification = 78
                end
                
                8 : begin
                sState.sMagnification = 3
                end
                
                 9 : begin
                sState.sMagnification = 9
                end
                
                10 : begin
                sState.sMagnification = 17
                end
                
                11 : begin
                sState.sMagnification = 35
                end
                
            endcase     ;  of listValue

			WIDGET_CONTROL, sState.wPixelXField, GET_VALUE=xPixelSizeStr
			WIDGET_CONTROL, sState.wPixelYField, GET_VALUE=yPixelSizeStr

			xPixelSize = DOUBLE(xPixelSizeStr[0])
			yPixelSize = DOUBLE(yPixelSizeStr[0])

			str = string(DOUBLE(sState.sX[sState.sCounter])*xPixelSize/DOUBLE(sState.sMagnification))
			WIDGET_CONTROL, sState.wXrLabel, SET_VALUE=str, /DYNAMIC_RESIZE

		   	str = string(DOUBLE(sState.sY[sState.sCounter])*yPixelSize/DOUBLE(sState.sMagnification))
			WIDGET_CONTROL, sState.wYrLabel, SET_VALUE=str, /DYNAMIC_RESIZE

		end

        'SLIDER' : begin

			if(sState.sElements gt 0) then begin

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Get image number

				WIDGET_CONTROL, sState.wSlider, GET_VALUE=i

				WIDGET_CONTROL, sState.wPixelXField, GET_VALUE=xPixelSizeStr
				WIDGET_CONTROL, sState.wPixelYField, GET_VALUE=yPixelSizeStr

				xPixelSize = DOUBLE(xPixelSizeStr[0])
				yPixelSize = DOUBLE(yPixelSizeStr[0])

				if(i lt sState.sElements) then begin

					sState.sCounter = i

					str = string(sState.sTimes[sState.sCounter])
					WIDGET_CONTROL, sState.wTimeLabel, SET_VALUE=str, /DYNAMIC_RESIZE

					str = string(sState.sX[sState.sCounter])
					WIDGET_CONTROL, sState.wXLabel, SET_VALUE=str, /DYNAMIC_RESIZE

					str = string(sState.sY[sState.sCounter])
					WIDGET_CONTROL, sState.wYLabel, SET_VALUE=str, /DYNAMIC_RESIZE

					str = string(DOUBLE(sState.sX[sState.sCounter])*xPixelSize/DOUBLE(sState.sMagnification))
					WIDGET_CONTROL, sState.wXrLabel, SET_VALUE=str, /DYNAMIC_RESIZE

				   	str = string(DOUBLE(sState.sY[sState.sCounter])*yPixelSize/DOUBLE(sState.sMagnification))
					WIDGET_CONTROL, sState.wYrLabel, SET_VALUE=str, /DYNAMIC_RESIZE

; End: Get image number
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Read image

					imageName = sState.sFiles[i]

					queryStatus = QUERY_IMAGE(imageName, imageInfo)

					imageSize = imageInfo.dimensions

					; Read image file
   					READ_JPEG, imageName, imageRGBData, TRUE=1
   					;imageData = REFORM(imageRGBData[0, *, *])

; End: Read image
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

					if sState.sX[i] gt 2 AND sState.sY[i] gt 2 then begin

						imageRGBData[0, sState.sX[i], sState.sY[i]] = 255
						imageRGBData[1, sState.sX[i], sState.sY[i]] = 255
						imageRGBData[2, sState.sX[i], sState.sY[i]] = 0

						imageRGBData[0, sState.sX[i]-1, sState.sY[i]] = 255
						imageRGBData[1, sState.sX[i]-1, sState.sY[i]] = 255
						imageRGBData[2, sState.sX[i]-1, sState.sY[i]] = 0

						imageRGBData[0, sState.sX[i]+1, sState.sY[i]] = 255
						imageRGBData[1, sState.sX[i]+1, sState.sY[i]] = 255
						imageRGBData[2, sState.sX[i]+1, sState.sY[i]] = 0

						imageRGBData[0, sState.sX[i]-2, sState.sY[i]] = 255
						imageRGBData[1, sState.sX[i]-2, sState.sY[i]] = 255
						imageRGBData[2, sState.sX[i]-2, sState.sY[i]] = 0

						imageRGBData[0, sState.sX[i]+2, sState.sY[i]] = 255
						imageRGBData[1, sState.sX[i]+2, sState.sY[i]] = 255
						imageRGBData[2, sState.sX[i]+2, sState.sY[i]] = 0

						imageRGBData[0, sState.sX[i], sState.sY[i]-1] = 255
						imageRGBData[1, sState.sX[i], sState.sY[i]-1] = 255
						imageRGBData[2, sState.sX[i], sState.sY[i]-1] = 0

						imageRGBData[0, sState.sX[i], sState.sY[i]+1] = 255
						imageRGBData[1, sState.sX[i], sState.sY[i]+1] = 255
						imageRGBData[2, sState.sX[i], sState.sY[i]+1] = 0

						imageRGBData[0, sState.sX[i], sState.sY[i]-2] = 255
						imageRGBData[1, sState.sX[i], sState.sY[i]-2] = 255
						imageRGBData[2, sState.sX[i], sState.sY[i]-2] = 0

						imageRGBData[0, sState.sX[i], sState.sY[i]+2] = 255
						imageRGBData[1, sState.sX[i], sState.sY[i]+2] = 255
						imageRGBData[2, sState.sX[i], sState.sY[i]+2] = 0
					endif

				if sState.cX ge 2 AND sState.cY ge 2 then begin

					imageRGBData[0, sState.cX, sState.cY] = 255
					imageRGBData[1, sState.cX, sState.cY] = 255
					imageRGBData[2, sState.cX, sState.cY] = 255

					imageRGBData[0, sState.cX-1, sState.cY] = 255
					imageRGBData[1, sState.cX-1, sState.cY] = 255
					imageRGBData[2, sState.cX-1, sState.cY] = 255

					imageRGBData[0, sState.cX+1, sState.cY] = 255
					imageRGBData[1, sState.cX+1, sState.cY] = 255
					imageRGBData[2, sState.cX+1, sState.cY] = 255

					imageRGBData[0, sState.cX-2, sState.cY] = 255
					imageRGBData[1, sState.cX-2, sState.cY] = 255
					imageRGBData[2, sState.cX-2, sState.cY] = 255

					imageRGBData[0, sState.cX+2, sState.cY] = 255
					imageRGBData[1, sState.cX+2, sState.cY] = 255
					imageRGBData[2, sState.cX+2, sState.cY] = 255

					imageRGBData[0, sState.cX, sState.cY-1] = 255
					imageRGBData[1, sState.cX, sState.cY-1] = 255
					imageRGBData[2, sState.cX, sState.cY-1] = 255

					imageRGBData[0, sState.cX, sState.cY+1] = 255
					imageRGBData[1, sState.cX, sState.cY+1] = 255
					imageRGBData[2, sState.cX, sState.cY+1] = 255

					imageRGBData[0, sState.cX, sState.cY-2] = 255
					imageRGBData[1, sState.cX, sState.cY-2] = 255
					imageRGBData[2, sState.cX, sState.cY-2] = 255

					imageRGBData[0, sState.cX, sState.cY+2] = 255
					imageRGBData[1, sState.cX, sState.cY+2] = 255
					imageRGBData[2, sState.cX, sState.cY+2] = 255

				endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Show image

					; Renew image of the view
					sState.oImage->SetProperty, DATA=imageRGBData, /NO_COPY

					; Redraw image view
					trapsDraw, sState
				endif

			endif

; End: Show image
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        end    ;  of SLIDER

        'DRAW': begin

            ;  Expose.
            ;
            if (sEvent.type eq 4) then begin
                trapsDraw, sState
                WIDGET_CONTROL, sEvent.top, SET_UVALUE=sState, /NO_COPY
                RETURN
            endif

            if (sEvent.press eq 4) then begin

				sState.cX = sEvent.x
				sState.cY = sEvent.y

			if(sState.sElements gt 0) then begin

					i = sState.sCounter
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Read image

					imageName = sState.sFiles[i]

					queryStatus = QUERY_IMAGE(imageName, imageInfo)

					imageSize = imageInfo.dimensions

					; Read image file
   					READ_JPEG, imageName, imageRGBData, TRUE=1
   					;imageData = REFORM(imageRGBData[0, *, *])

; End: Read image
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

					if sState.sX[i] gt 2 AND sState.sY[i] gt 2 then begin

						imageRGBData[0, sState.sX[i], sState.sY[i]] = 255
						imageRGBData[1, sState.sX[i], sState.sY[i]] = 255
						imageRGBData[2, sState.sX[i], sState.sY[i]] = 0

						imageRGBData[0, sState.sX[i]-1, sState.sY[i]] = 255
						imageRGBData[1, sState.sX[i]-1, sState.sY[i]] = 255
						imageRGBData[2, sState.sX[i]-1, sState.sY[i]] = 0

						imageRGBData[0, sState.sX[i]+1, sState.sY[i]] = 255
						imageRGBData[1, sState.sX[i]+1, sState.sY[i]] = 255
						imageRGBData[2, sState.sX[i]+1, sState.sY[i]] = 0

						imageRGBData[0, sState.sX[i]-2, sState.sY[i]] = 255
						imageRGBData[1, sState.sX[i]-2, sState.sY[i]] = 255
						imageRGBData[2, sState.sX[i]-2, sState.sY[i]] = 0

						imageRGBData[0, sState.sX[i]+2, sState.sY[i]] = 255
						imageRGBData[1, sState.sX[i]+2, sState.sY[i]] = 255
						imageRGBData[2, sState.sX[i]+2, sState.sY[i]] = 0

						imageRGBData[0, sState.sX[i], sState.sY[i]-1] = 255
						imageRGBData[1, sState.sX[i], sState.sY[i]-1] = 255
						imageRGBData[2, sState.sX[i], sState.sY[i]-1] = 0

						imageRGBData[0, sState.sX[i], sState.sY[i]+1] = 255
						imageRGBData[1, sState.sX[i], sState.sY[i]+1] = 255
						imageRGBData[2, sState.sX[i], sState.sY[i]+1] = 0

						imageRGBData[0, sState.sX[i], sState.sY[i]-2] = 255
						imageRGBData[1, sState.sX[i], sState.sY[i]-2] = 255
						imageRGBData[2, sState.sX[i], sState.sY[i]-2] = 0

						imageRGBData[0, sState.sX[i], sState.sY[i]+2] = 255
						imageRGBData[1, sState.sX[i], sState.sY[i]+2] = 255
						imageRGBData[2, sState.sX[i], sState.sY[i]+2] = 0
					endif

				if sState.cX ge 2 AND sState.cY ge 2 then begin

					imageRGBData[0, sState.cX, sState.cY] = 255
					imageRGBData[1, sState.cX, sState.cY] = 255
					imageRGBData[2, sState.cX, sState.cY] = 255

					imageRGBData[0, sState.cX-1, sState.cY] = 255
					imageRGBData[1, sState.cX-1, sState.cY] = 255
					imageRGBData[2, sState.cX-1, sState.cY] = 255

					imageRGBData[0, sState.cX+1, sState.cY] = 255
					imageRGBData[1, sState.cX+1, sState.cY] = 255
					imageRGBData[2, sState.cX+1, sState.cY] = 255

					imageRGBData[0, sState.cX-2, sState.cY] = 255
					imageRGBData[1, sState.cX-2, sState.cY] = 255
					imageRGBData[2, sState.cX-2, sState.cY] = 255

					imageRGBData[0, sState.cX+2, sState.cY] = 255
					imageRGBData[1, sState.cX+2, sState.cY] = 255
					imageRGBData[2, sState.cX+2, sState.cY] = 255

					imageRGBData[0, sState.cX, sState.cY-1] = 255
					imageRGBData[1, sState.cX, sState.cY-1] = 255
					imageRGBData[2, sState.cX, sState.cY-1] = 255

					imageRGBData[0, sState.cX, sState.cY+1] = 255
					imageRGBData[1, sState.cX, sState.cY+1] = 255
					imageRGBData[2, sState.cX, sState.cY+1] = 255

					imageRGBData[0, sState.cX, sState.cY-2] = 255
					imageRGBData[1, sState.cX, sState.cY-2] = 255
					imageRGBData[2, sState.cX, sState.cY-2] = 255

					imageRGBData[0, sState.cX, sState.cY+2] = 255
					imageRGBData[1, sState.cX, sState.cY+2] = 255
					imageRGBData[2, sState.cX, sState.cY+2] = 255

				endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Show image

					; Renew image of the view
					sState.oImage->SetProperty, DATA=imageRGBData, /NO_COPY

					; Redraw image view
					trapsDraw, sState
				endif

			endif
		; Button pressed
        IF (sEvent.type EQ 0) THEN BEGIN
         	if (sEvent.press eq 1) then begin

				sState.btndown = 1b

				sState.XPos = sEvent.x
				sState.YPos = sEvent.y

			endif
		endif

		; Button released
      	IF (sEvent.type EQ 1) THEN BEGIN
            IF (sState.btndown EQ 1b) THEN BEGIN
				sState.XDelta = abs(sEvent.x - sState.XPos)
				sState.YDelta = abs(sEvent.y - sState.YPos)

				if(sState.XDelta lt 2) then sState.XDelta = 2
				if(sState.YDelta lt 2) then sState.YDelta = 2

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Read image

					centerX = sState.XPos
					centerY = sState.YPos

					i=sState.sCounter
;sState.sElements = 10
					for i=sState.sCounter, (sState.sElements)-1 do begin
						imageName = sState.sFiles[i]

						; Read image file
   						READ_JPEG, imageName, imageRGBData, TRUE=1
   						imageData = REFORM(imageRGBData[1, *, *])

						trapDetection, imageRGBData, sState.xdim, sState.ydim, centerX, centerY, sState.XDelta, sState.YDelta, sState.sFilterValue, bubbleX, bubbleY

						if bubbleX eq 0 OR bubbleY eq 0 then begin

							if i gt 0 then begin
								sState.sX[i] = sState.sX[i-1]
								sState.sY[i] = sState.sY[i-1]
							endif else begin
								sState.sX[i] = bubbleX
								sState.sY[i] = bubbleY
							endelse

						endif else begin
							sState.sX[i] = bubbleX
							sState.sY[i] = bubbleY
						endelse

						print, i, sState.sX[i], sState.sY[i]

						centerX = sState.sX[i]
						centerY = sState.sY[i]

					endfor

; End: Read image
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
				imageName = sState.sFiles[sState.sCounter]

				WIDGET_CONTROL, sState.wPixelXField, GET_VALUE=xPixelSizeStr
				WIDGET_CONTROL, sState.wPixelYField, GET_VALUE=yPixelSizeStr
				WIDGET_CONTROL, sState.wSaveAsButton, SENSITIVE=1

				xPixelSize = DOUBLE(xPixelSizeStr[0])
				yPixelSize = DOUBLE(yPixelSizeStr[0])

				queryStatus = QUERY_IMAGE(imageName, imageInfo)

				imageSize = imageInfo.dimensions

				; Read image file
   				READ_JPEG, imageName, imageRGBData, TRUE=1

				i = sState.sCounter

					imageRGBData[0, sState.sX[i], sState.sY[i]] = 255
					imageRGBData[1, sState.sX[i], sState.sY[i]] = 255
					imageRGBData[2, sState.sX[i], sState.sY[i]] = 0

					imageRGBData[0, sState.sX[i]-1, sState.sY[i]] = 255
					imageRGBData[1, sState.sX[i]-1, sState.sY[i]] = 255
					imageRGBData[2, sState.sX[i]-1, sState.sY[i]] = 0

					imageRGBData[0, sState.sX[i]+1, sState.sY[i]] = 255
					imageRGBData[1, sState.sX[i]+1, sState.sY[i]] = 255
					imageRGBData[2, sState.sX[i]+1, sState.sY[i]] = 0

					imageRGBData[0, sState.sX[i]-2, sState.sY[i]] = 255
					imageRGBData[1, sState.sX[i]-2, sState.sY[i]] = 255
					imageRGBData[2, sState.sX[i]-2, sState.sY[i]] = 0

					imageRGBData[0, sState.sX[i]+2, sState.sY[i]] = 255
					imageRGBData[1, sState.sX[i]+2, sState.sY[i]] = 255
					imageRGBData[2, sState.sX[i]+2, sState.sY[i]] = 0

					imageRGBData[0, sState.sX[i], sState.sY[i]-1] = 255
					imageRGBData[1, sState.sX[i], sState.sY[i]-1] = 255
					imageRGBData[2, sState.sX[i], sState.sY[i]-1] = 0

					imageRGBData[0, sState.sX[i], sState.sY[i]+1] = 255
					imageRGBData[1, sState.sX[i], sState.sY[i]+1] = 255
					imageRGBData[2, sState.sX[i], sState.sY[i]+1] = 0

					imageRGBData[0, sState.sX[i], sState.sY[i]-2] = 255
					imageRGBData[1, sState.sX[i], sState.sY[i]-2] = 255
					imageRGBData[2, sState.sX[i], sState.sY[i]-2] = 0

					imageRGBData[0, sState.sX[i], sState.sY[i]+2] = 255
					imageRGBData[1, sState.sX[i], sState.sY[i]+2] = 255
					imageRGBData[2, sState.sX[i], sState.sY[i]+2] = 0


				if sState.cX[i] ge 2 AND sState.cY[i] ge 2 then begin

					imageRGBData[0, sState.cX, sState.cY] = 255
					imageRGBData[1, sState.cX, sState.cY] = 255
					imageRGBData[2, sState.cX, sState.cY] = 255

					imageRGBData[0, sState.cX-1, sState.cY] = 255
					imageRGBData[1, sState.cX-1, sState.cY] = 255
					imageRGBData[2, sState.cX-1, sState.cY] = 255

					imageRGBData[0, sState.cX+1, sState.cY] = 255
					imageRGBData[1, sState.cX+1, sState.cY] = 255
					imageRGBData[2, sState.cX+1, sState.cY] = 255

					imageRGBData[0, sState.cX-2, sState.cY] = 255
					imageRGBData[1, sState.cX-2, sState.cY] = 255
					imageRGBData[2, sState.cX-2, sState.cY] = 255

					imageRGBData[0, sState.cX+2, sState.cY] = 255
					imageRGBData[1, sState.cX+2, sState.cY] = 255
					imageRGBData[2, sState.cX+2, sState.cY] = 255

					imageRGBData[0, sState.cX, sState.cY-1] = 255
					imageRGBData[1, sState.cX, sState.cY-1] = 255
					imageRGBData[2, sState.cX, sState.cY-1] = 255

					imageRGBData[0, sState.cX, sState.cY+1] = 255
					imageRGBData[1, sState.cX, sState.cY+1] = 255
					imageRGBData[2, sState.cX, sState.cY+1] = 255

					imageRGBData[0, sState.cX, sState.cY-2] = 255
					imageRGBData[1, sState.cX, sState.cY-2] = 255
					imageRGBData[2, sState.cX, sState.cY-2] = 255

					imageRGBData[0, sState.cX, sState.cY+2] = 255
					imageRGBData[1, sState.cX, sState.cY+2] = 255
					imageRGBData[2, sState.cX, sState.cY+2] = 255

				endif

				; Renew image of the view
				sState.oImage->SetProperty, DATA=imageRGBData, /NO_COPY

				str = string(sState.sX[sState.sCounter])
				WIDGET_CONTROL, sState.wXLabel, SET_VALUE=str, /DYNAMIC_RESIZE

				str = string(sState.sY[sState.sCounter])
				WIDGET_CONTROL, sState.wYLabel, SET_VALUE=str, /DYNAMIC_RESIZE

				str = string(DOUBLE(sState.sX[sState.sCounter])*xPixelSize/DOUBLE(sState.sMagnification))
				WIDGET_CONTROL, sState.wXrLabel, SET_VALUE=str, /DYNAMIC_RESIZE

			   	str = string(DOUBLE(sState.sY[sState.sCounter])*yPixelSize/DOUBLE(sState.sMagnification))
				WIDGET_CONTROL, sState.wYrLabel, SET_VALUE=str, /DYNAMIC_RESIZE

				; Redraw image view
				trapsDraw, sState

            endif
            sState.btndown = 0b
        endif

            if (sEvent.press eq -10) then begin
            	if(sState.sElements gt 1) then begin
					sState.XPos = sEvent.x
					sState.YPos = sEvent.y

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Read image

					centerX = sState.XPos
					centerY = sState.YPos

					i=sState.sCounter

					for i=sState.sCounter, (sState.sElements)-1 do begin
						imageName = sState.sFiles[i]

						; Read image file
   						READ_JPEG, imageName, imageRGBData, TRUE=1
   						imageData = REFORM(imageRGBData[1, *, *])

						trapDetection, imageRGBData, sState.xdim, sState.ydim, centerX, centerY, sState.XDelta, sState.YDelta, sState.sFilterValue, bubbleX, bubbleY

						if bubbleX eq 0 OR bubbleY eq 0 then begin

							if i gt 0 then begin
								sState.sX[i] = sState.sX[i-1]
								sState.sY[i] = sState.sY[i-1]
							endif else begin
								sState.sX[i] = bubbleX
								sState.sY[i] = bubbleY
							endelse

						endif else begin
							sState.sX[i] = bubbleX
							sState.sY[i] = bubbleY
						endelse

						print, i, sState.sX[i], sState.sY[i]

						centerX = sState.sX[i]
						centerY = sState.sY[i]

					endfor

; End: Read image
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;				endif

				imageName = sState.sFiles[sState.sCounter]

				WIDGET_CONTROL, sState.wPixelXField, GET_VALUE=xPixelSizeStr
				WIDGET_CONTROL, sState.wPixelYField, GET_VALUE=yPixelSizeStr
				WIDGET_CONTROL, sState.wSaveAsButton, SENSITIVE=1

				xPixelSize = DOUBLE(xPixelSizeStr[0])
				yPixelSize = DOUBLE(yPixelSizeStr[0])

				queryStatus = QUERY_IMAGE(imageName, imageInfo)

				imageSize = imageInfo.dimensions

				; Read image file
   				READ_JPEG, imageName, imageRGBData, TRUE=1

				i = sState.sCounter

					imageRGBData[0, sState.sX[i], sState.sY[i]] = 255
					imageRGBData[1, sState.sX[i], sState.sY[i]] = 255
					imageRGBData[2, sState.sX[i], sState.sY[i]] = 0

					imageRGBData[0, sState.sX[i]-1, sState.sY[i]] = 255
					imageRGBData[1, sState.sX[i]-1, sState.sY[i]] = 255
					imageRGBData[2, sState.sX[i]-1, sState.sY[i]] = 0

					imageRGBData[0, sState.sX[i]+1, sState.sY[i]] = 255
					imageRGBData[1, sState.sX[i]+1, sState.sY[i]] = 255
					imageRGBData[2, sState.sX[i]+1, sState.sY[i]] = 0

					imageRGBData[0, sState.sX[i]-2, sState.sY[i]] = 255
					imageRGBData[1, sState.sX[i]-2, sState.sY[i]] = 255
					imageRGBData[2, sState.sX[i]-2, sState.sY[i]] = 0

					imageRGBData[0, sState.sX[i]+2, sState.sY[i]] = 255
					imageRGBData[1, sState.sX[i]+2, sState.sY[i]] = 255
					imageRGBData[2, sState.sX[i]+2, sState.sY[i]] = 0

					imageRGBData[0, sState.sX[i], sState.sY[i]-1] = 255
					imageRGBData[1, sState.sX[i], sState.sY[i]-1] = 255
					imageRGBData[2, sState.sX[i], sState.sY[i]-1] = 0

					imageRGBData[0, sState.sX[i], sState.sY[i]+1] = 255
					imageRGBData[1, sState.sX[i], sState.sY[i]+1] = 255
					imageRGBData[2, sState.sX[i], sState.sY[i]+1] = 0

					imageRGBData[0, sState.sX[i], sState.sY[i]-2] = 255
					imageRGBData[1, sState.sX[i], sState.sY[i]-2] = 255
					imageRGBData[2, sState.sX[i], sState.sY[i]-2] = 0

					imageRGBData[0, sState.sX[i], sState.sY[i]+2] = 255
					imageRGBData[1, sState.sX[i], sState.sY[i]+2] = 255
					imageRGBData[2, sState.sX[i], sState.sY[i]+2] = 0


				if sState.cX[i] ge 2 AND sState.cY[i] ge 2 then begin

					imageRGBData[0, sState.cX, sState.cY] = 255
					imageRGBData[1, sState.cX, sState.cY] = 255
					imageRGBData[2, sState.cX, sState.cY] = 255

					imageRGBData[0, sState.cX-1, sState.cY] = 255
					imageRGBData[1, sState.cX-1, sState.cY] = 255
					imageRGBData[2, sState.cX-1, sState.cY] = 255

					imageRGBData[0, sState.cX+1, sState.cY] = 255
					imageRGBData[1, sState.cX+1, sState.cY] = 255
					imageRGBData[2, sState.cX+1, sState.cY] = 255

					imageRGBData[0, sState.cX-2, sState.cY] = 255
					imageRGBData[1, sState.cX-2, sState.cY] = 255
					imageRGBData[2, sState.cX-2, sState.cY] = 255

					imageRGBData[0, sState.cX+2, sState.cY] = 255
					imageRGBData[1, sState.cX+2, sState.cY] = 255
					imageRGBData[2, sState.cX+2, sState.cY] = 255

					imageRGBData[0, sState.cX, sState.cY-1] = 255
					imageRGBData[1, sState.cX, sState.cY-1] = 255
					imageRGBData[2, sState.cX, sState.cY-1] = 255

					imageRGBData[0, sState.cX, sState.cY+1] = 255
					imageRGBData[1, sState.cX, sState.cY+1] = 255
					imageRGBData[2, sState.cX, sState.cY+1] = 255

					imageRGBData[0, sState.cX, sState.cY-2] = 255
					imageRGBData[1, sState.cX, sState.cY-2] = 255
					imageRGBData[2, sState.cX, sState.cY-2] = 255

					imageRGBData[0, sState.cX, sState.cY+2] = 255
					imageRGBData[1, sState.cX, sState.cY+2] = 255
					imageRGBData[2, sState.cX, sState.cY+2] = 255

				endif

				; Renew image of the view
				sState.oImage->SetProperty, DATA=imageRGBData, /NO_COPY

				str = string(sState.sX[sState.sCounter])
				WIDGET_CONTROL, sState.wXLabel, SET_VALUE=str, /DYNAMIC_RESIZE

				str = string(sState.sY[sState.sCounter])
				WIDGET_CONTROL, sState.wYLabel, SET_VALUE=str, /DYNAMIC_RESIZE

				str = string(DOUBLE(sState.sX[sState.sCounter])*xPixelSize/DOUBLE(sState.sMagnification))
				WIDGET_CONTROL, sState.wXrLabel, SET_VALUE=str, /DYNAMIC_RESIZE

			   	str = string(DOUBLE(sState.sY[sState.sCounter])*yPixelSize/DOUBLE(sState.sMagnification))
				WIDGET_CONTROL, sState.wYrLabel, SET_VALUE=str, /DYNAMIC_RESIZE

				; Redraw image view
				trapsDraw, sState

				endif
			endif

        end                 ;     of DRAW

        "ABOUT": begin

			sState.sCounter = 1

           	void = DIALOG_MESSAGE(['Traps Application v0.2', $
                                 'Created by Sasha Skydan', $
                                 'o.skydan@gmail.com'], TITLE='About Information')
        end

        "SAVEAS": begin

			WIDGET_CONTROL, sState.wPixelXField, GET_VALUE=xPixelSizeStr
			WIDGET_CONTROL, sState.wPixelYField, GET_VALUE=yPixelSizeStr

			xPixelSize = DOUBLE(xPixelSizeStr[0])
			yPixelSize = DOUBLE(yPixelSizeStr[0])

			dataFile = DIALOG_PICKFILE(TITLE="Save XY data.")

			OPENW, 1, dataFile

			PRINTF, 1, 'Frames  ', 'Time, sec          ', 'X, px     ', 'Y, px     ', 'X, um          ', 'Y, um          '

			for i=0, (sState.sElements)-1 do begin

				pointX = sState.sX[i] - sState.cX
				pointY = sState.sY[i] - sState.cY

				;strX = string(DOUBLE(sState.sX[i])*xPixelSize/DOUBLE(sState.sMagnification))
				;strY = string(DOUBLE(sState.sY[i])*yPixelSize/DOUBLE(sState.sMagnification))
				;PRINTF, 1, i, sState.sTimes[i], sState.sX[i]+sState.cX, sState.sY[i]+sState.cY, strX, strY

				strX = string(DOUBLE(pointX)*xPixelSize/DOUBLE(sState.sMagnification))
				strY = string(DOUBLE(pointY)*yPixelSize/DOUBLE(sState.sMagnification))

				PRINTF, 1, i, sState.sTimes[i], pointX, pointY, strX, strY
       
			ENDFOR

			CLOSE, 1

        end

        "OPEN": begin

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Read folder of the frames
			WIDGET_CONTROL, sState.wSaveAsButton, SENSITIVE=0

			sState.sTimes[*] = 0
			sState.sFiles[*] = ''
			sState.sX[*] = 0
			sState.sX[*] = 0
			sState.sElements = 0
			sState.sCounter = 0

			fFolder = DIALOG_PICKFILE(/DIRECTORY, TITLE="Choose directory containing video frames data.")

			fFiles = FILE_SEARCH(fFolder, '*.jpg', /FOLD_CASE)

			fElements = 0
			; Get number of files in the folder
			fElements = N_ELEMENTS(fFiles)

			if(fElements gt 1) then begin

				; Array, which contains time of the frame
				fTimes = dblarr(fElements)

				; Get file name and transform it into the time
				FOR i=0, fElements-1 DO BEGIN
					fTimes[i] = double( file_basename(fFiles[i],'.jpg'))
				ENDFOR

				; Get sorting index for sorting procedure
				sIndex = sort(fTimes)

				; Run sorting procedure
				sState.sTimes = fTimes[sIndex]
				sState.sFiles = fFiles[sIndex]

				; Save a number of frames for processing
				sState.sElements = fElements

				; Save current frame number to 0
				sState.sCounter = 0

				; Correct the slider and label configurations
				WIDGET_CONTROL, sState.wSlider, SET_VALUE=sState.sCounter
				WIDGET_CONTROL, sState.wSlider, SET_SLIDER_MAX=(sState.sElements-1)

				str = string(sState.sTimes[0])
				WIDGET_CONTROL, sState.wTimeLabel, SET_VALUE=str, /DYNAMIC_RESIZE

; End: Read folder of the frames
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Read image

				imageName = sState.sFiles[0]

				queryStatus = QUERY_IMAGE(imageName, imageInfo)

				imageSize = imageInfo.dimensions

				; Read image file
   				READ_JPEG, imageName, imageRGBData, TRUE=1
   				imageData = REFORM(imageRGBData[1, *, *])

				;sState.xDim = imageSize[0]
				;sState.yDim = imageSize[1]

				;trapsDetection, imageRGBData, sState.xdim, sState.ydim, sState.sFilterValue

; End: Read image
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Show image

				; Renew image of the view
				sState.oImage->SetProperty, DATA=imageRGBData, /NO_COPY

				WIDGET_CONTROL, sState.wTimeLabel, SET_VALUE='0', /DYNAMIC_RESIZE
				WIDGET_CONTROL, sState.wXLabel, SET_VALUE='0', /DYNAMIC_RESIZE
				WIDGET_CONTROL, sState.wYLabel, SET_VALUE='0', /DYNAMIC_RESIZE

				; Redraw image view
				trapsDraw, sState

			endif else begin

				; Save a number of frames to 0
				sState.sElements = 0

				; Save current frame number to 0
				sState.sCounter = 0

				; Correct the slider and label configurations
				WIDGET_CONTROL, sState.wSlider, SET_VALUE=sState.sCounter
				WIDGET_CONTROL, sState.wSlider, SET_SLIDER_MAX=(sState.sElements+1)

				WIDGET_CONTROL, sState.wTimeLabel, SET_VALUE='', /DYNAMIC_RESIZE

			endelse

; End: Show image
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

        end

        "QUIT": begin

            ;  Destroy widget hierarchy.
            ;
            WIDGET_CONTROL, sEvent.top, /DESTROY

            RETURN
        end

        ELSE :  begin
            PRINT, 'Case Statement found no matches'
        end

    endcase

	WIDGET_CONTROL, sEvent.top, Set_UValue=sState, /No_Copy

end               ; of d_widgetsEvent

; -----------------------------------------------------------------------------
;
;  Purpose:  Cleanup procedure
;
pro trapsCleanup, $
    wTopBase      ; IN: top level base associated with the cleanup

    ;  Get the color table saved in the window's user value.
    ;
    WIDGET_CONTROL, wTopBase, GET_UVALUE=sState,/No_Copy

end   ; of d_widgetsCleanup

; -----------------------------------------------------------------------------
;
;  Purpose:  Main procedure of the widgets demo
;
pro traps, $

    GROUP=group, $     ; IN: (opt) group identifier
    RECORD_TO_FILENAME=record_to_filename, $
    DEBUG=debug, $     ; IN: (opt)
    APPTLB = appTLB    ; OUT: (opt) TLB of this application

    ; Check the validity of the group identifier
    ;
    ngroup = N_ELEMENTS(group)
    if (ngroup NE 0) then begin
        check = WIDGET_INFO(group, /VALID_ID)
        if (check NE 1) then begin
            print,'Error, the group identifier is not valid'
            print, 'Return to the main application'
            RETURN
        endif
        groupBase = group
    endif else groupBase = 0L


    ;  Get the screen size.
    ;
    Device, GET_SCREEN_SIZE = screenSize

    ;  Make the system have a maximum of 256 colors
    ;
    numcolors = !d.N_COLORS
    if( (( !D.NAME EQ 'X') or (!D.NAME EQ 'MAC')) $
        and (!d.N_COLORS GE 256L)) then $
        DEVICE, PSEUDO_COLOR=8
    DEVICE, DECOMPOSED=0, BYPASS_TRANSLATION=0

    ;  Get the current color table
    ;
    TVLCT, savedR, savedG, savedB, /GET

    ;  Build color table from color vectors
    ;
    colorTable = [[savedR],[savedG],[savedB]]

	; Set image resolution
	xDim =512
	yDim = 512

    ; Define a main widget base.
    ;
    wTopBase = WIDGET_BASE(TITLE="Widgets", /COLUMN, $
            /TLB_KILL_REQUEST_EVENTS, $
            TLB_FRAME_ATTR=1, MBAR=barBase)

    	;  Create the quit button
    	;
    	wFileButton = WIDGET_BUTTON(barBase, VALUE= 'File', /MENU)

		wOpenButton = WIDGET_BUTTON(wFileButton, VALUE='Open', UVALUE='OPEN')

		wSaveAsButton = WIDGET_BUTTON(wFileButton, VALUE='Save As...', UVALUE='SAVEAS', SENSITIVE=0)

    	wQuitButton = WIDGET_BUTTON(wFileButton, VALUE='Quit', UVALUE='QUIT')


    	wActionsButton = WIDGET_BUTTON(barBase, VALUE='Actions', /MENU)

    	wZEROButton = WIDGET_BUTTON(wActionsButton, VALUE='Set 0 point', UVALUE='ZERO')


    	; Create the help button
    	;
    	wHelpButton = WIDGET_BUTTON(barBase, /HELP, VALUE='About', /MENU)

    	wAboutButton = WIDGET_BUTTON(wHelpButton, VALUE='About Traps', UVALUE='ABOUT')

    ;  Create the first child of the graghic base.
    ;
    wSubBase = WIDGET_BASE(wTopBase, COLUMN=2)

        wSliderBase = WIDGET_BASE(wSubBase, /COLUMN, /FRAME, YPAD=10, XPAD=22)

        	; Initial rotation values, also used below when
            ; initializing model
            ;frameNum = 0
            wSliderLabel = WIDGET_LABEL(wSliderBase, $
                        VALUE='Frame Number', /ALIGN_CENTER)

            wSlider = WIDGET_SLIDER(wSliderBase, $
                        UVALUE='SLIDER', $
                        VALUE=frameNum, MINIMUM=0, MAXIMUM=1)

            wTimeTitleLabel = WIDGET_LABEL(wSliderBase, VALUE='Time, sec', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            wTimeLabel = WIDGET_LABEL(wSliderBase, VALUE='0', /ALIGN_LEFT, /DYNAMIC_RESIZE)

			fLabel0 = WIDGET_LABEL(wSliderBase, VALUE=' ', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            wXTitleLabel = WIDGET_LABEL(wSliderBase, VALUE='X pos, px', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            wXLabel = WIDGET_LABEL(wSliderBase, VALUE='0', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            wYTitleLabel = WIDGET_LABEL(wSliderBase, VALUE='Y pos, px', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            wYLabel = WIDGET_LABEL(wSliderBase, VALUE='0', /ALIGN_LEFT, /DYNAMIC_RESIZE)


            wXrTitleLabel = WIDGET_LABEL(wSliderBase, VALUE='X pos, um', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            wXrLabel = WIDGET_LABEL(wSliderBase, VALUE='0', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            wYrTitleLabel = WIDGET_LABEL(wSliderBase, VALUE='Y pos, um', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            wYrLabel = WIDGET_LABEL(wSliderBase, VALUE='0', /ALIGN_LEFT, /DYNAMIC_RESIZE)


        wInfoBase = WIDGET_BASE(wSubBase, /COLUMN, /FRAME, YPAD=10, XPAD=22)

            fTitleLabel = WIDGET_LABEL(wInfoBase, VALUE='Threashold value', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            fSlider = WIDGET_SLIDER(wInfoBase, VALUE=60, MINIMUM=0, MAXIMUM=255, UVALUE='SLIDER_FILTER')

			fFactorLabel1 = WIDGET_LABEL(wInfoBase, VALUE=' ', /ALIGN_LEFT, /DYNAMIC_RESIZE)
			fFactorLabel = WIDGET_LABEL(wInfoBase, VALUE='Magnification factor', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            fFactor = WIDGET_DROPLIST(wInfoBase, VALUE=['2', '5', '10', '20', '30', '40', '60', '78', '3', '9', '17', '35'], UVALUE='M_FACTOR')

			fLabel1 = WIDGET_LABEL(wInfoBase, VALUE=' ', /ALIGN_LEFT, /DYNAMIC_RESIZE)
			fPixelXLabel = WIDGET_LABEL(wInfoBase, VALUE='Pixel size in X, um', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            wPixelXField = widget_text(wInfoBase, /EDITABLE, UVALUE='PXTEXT', VALUE='10')

			fLabel2 = WIDGET_LABEL(wInfoBase, VALUE=' ', /ALIGN_LEFT, /DYNAMIC_RESIZE)
			fPixelYLabel = WIDGET_LABEL(wInfoBase, VALUE='Pixel size in y, um', /ALIGN_LEFT, /DYNAMIC_RESIZE)

            wPixelYField = widget_text(wInfoBase, /EDITABLE, UVALUE='PYTEXT', VALUE='10')


   ;  Create a widget for data drwawing
   ;
   wDraw = WIDGET_DRAW(wSubBase, $
                        XSIZE=xDim, YSIZE=yDim, $
                        /BUTTON_EVENTS, $
                        /EXPOSE_EVENTS, UVALUE='DRAW', $
                        RETAIN=0, $
                        GRAPHICS_LEVEL=2)

    ;  Realize the widget hierarchy.
    ;
    WIDGET_CONTROL, wTopBase, /REALIZE

    WIDGET_CONTROL, wTopBase, SENSITIVE=0

    ; Determine the window value of plot window, wDraw1.
    ;
    WIDGET_CONTROL, wDraw, GET_VALUE=drawWindowID

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	oView = OBJ_NEW('IDLgrView', VIEWPLANE_RECT = [0., 0., xDim, yDim], COLOR=[0,0,0])

    oModel = OBJ_NEW('IDLgrModel')

	oPalette = OBJ_NEW('IDLgrPalette')
	oPalette->LOADCT, 5

	;oImage = OBJ_NEW('IDLgrImage', redChannel, PALETTE = oPalette)
	oImage = OBJ_NEW('IDLgrImage', PALETTE = oPalette)

    oModel -> Add, oImage

    oView -> Add, oModel

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;  Create the info structure
    ;
    sState={ $
		btndown: 0b,           $
		sElements: 0, $
		sFiles: STRARR(5000),$
		sTimes: DBLARR(5000),$
		sX: INTARR(5000), $
		sY: INTARR(5000), $
		sCounter: 0, $			; Current image number
		sFilterValue: 60, $		; Threashold value for the filtering
		sMagnification: 2, $	; Magnification factor
		XDim: xDim, $			; Image resolution in the x direction
		YDim: yDim, $			; Image resolution in the y direction
		XPos: -1, $				; Initial position of the bubble entered by the user in the x direction
		YPos: -1, $				; Initial position of the bubble entered by the user in the y direction

		XDelta: 5, $				; Delta range in the x direction
		YDelta: 5, $				; Delta range in the y direction

		oImage: oImage, $
		OModel: oModel, $
        OView: oView, $                         ; View object
        DrawWindowID: drawWindowID, $           ; Window ID
        WDraw: wDraw, $                         ; Widget draw ID
        WSlider: wSlider, $
        FSlider: fSlider, $
        WTimeLabel: wTimeLabel, $
        WXLabel: wXLabel, $
        WYLabel: wYLabel, $
        WXrLabel: wXrLabel, $
        WYrLabel: wYrLabel, $
        WPixelXField: wPixelXField, $
        WPixelYField: wPixelYField, $
        WSaveAsButton: wSaveAsButton, $
        textureFlag: 1, $                       ; Texture mapping flag
        anotherFlag: 2, $                  ; Base of Group Leader
        cX: 0, $                  ; Correction for 0 of the X coordinates
        cY: 0 $                  ; Correction for 0 of the  coordinates
    }

    ;  Register the info structure in the user value of the top-level base
    ;
    WIDGET_CONTROL, wTopBase, SET_UVALUE=sState, /NO_COPY

    WIDGET_CONTROL, wTopBase, SENSITIVE=1

    ;  Map the top level base.
    ;
    WIDGET_CONTROL, wTopBase, MAP=1

    XMANAGER, "d_widgets", wTopBase, $
        /NO_BLOCK, $
        EVENT_HANDLER="trapsEvent", CLEANUP="trapsCleanup"

end   ;  main procedure
