pro ClipData1

;Clips out the unwanted data spikes from force data acquired from Optical_trap_force.pro
;Read the singlepath data from the file
OPENU, Unit,'C:\Users\Steven Ross\Desktop\XYDataForceResults\Tip 44 45 Insert 500\Optical_trap_force\XYForce1.dat',/GET_LUN, /NOEXPAND_PATH             
NUMFRAMES=30000L
READU, Unit, NUMFRAMES
print,"NUMFRAMES =",NUMFRAMES
X_um=FLTARR(NUMFRAMES)
Y_um=FLTARR(NUMFRAMES)
Force_x=FLTARR(NUMFRAMES)
Force_y=FLTARR(NUMFRAMES)
READU, Unit, X_um,Y_um,Force_x,Force_y
CLOSE, Unit
FREE_LUN, Unit

;Plot the combined paths
X_Mag=3.0
Y_Mag=3.0
X_Pix=INTARR(NUMFRAMES)
Y_Pix=INTARR(NUMFRAMES)
for i=0L,NUMFRAMES-1 do begin
  X_Pix[i]=FIX(X_um[i]*X_mag)
  Y_Pix[i]=FIX(Y_um[i]*Y_mag)
endfor

;Set up the reconstructed path image
Comb_Path_Image=BYTARR(512L,512L)

;Create the path image from the data - note origin shifted to centre point
for i=0L,NUMFRAMES-1 do begin
  x_coord=X_Pix[i]+255L
  y_coord=Y_Pix[i]+255L
  Comb_Path_Image[x_coord,y_coord]=255;
end

;Display the path
;WINDOW,24,TITLE="Combined Pixel Path",XSIZE=512,YSIZE=512
;TVSCL, Comb_Path_Image

;Calculate the resultant force
Res_Force=FLTARR(NUMFRAMES)
for i=0L,NUMFRAMES-1 do begin
  Res_Force[i]=SQRT(Force_x[i]^2+Force_y[i]^2)
endfor

;Calculate the distance travelled
Dist_trav=FLTARR(NUMFRAMES)
for i=0L,NUMFRAMES-1 do begin
  Dist_trav[i]=SQRT(X_um[i]^2.0+Y_um[i]^2.0)*1e-6
endfor
Result = size(Res_Force)
Print, Result
Disext = Size(Dist_trav)
print, "Dist_trav = ",Dist_trav

MAX_Dist_trav=Max(Dist_trav)
MIN_Dist_trav=Min(Dist_Trav)
Total_Dist_Trav=MAX_Dist_Trav-MIN_Dist_trav
  PRINT, 'Total Dist Trav =',Total_Dist_trav
max_X_um=MAX(X_um)
min_X_um=MIN(X_um)
max_Y_um=MAX(Y_um)
min_Y_um=MIN(Y_um)
max_force=MAX(Res_Force)
min_force=MIN(Res_Force)

;WINDOW,25,TITLE="Surface",XSIZE=1024,YSIZE=700
;TRIANGULATE, X_um,Y_um,tr,b
;SURFACE, TRIGRID(X_um,Y_um,Res_Force,tr, EXTRAPOLATE=b, /QUINTIC),MAX_VALUE=25.0e-12,CHARSIZE=2.0
;SURFACE, TRIGRID(X_um,Y_um,Res_Force,tr, EXTRAPOLATE=b, /QUINTIC),CHARSIZE=2.0

WINDOW,26,TITLE='Force Distance Plot ',XSIZE=720,YSIZE=720 ;put name of experiment in title i.e tip 92
PLOT,Dist_trav, Res_Force; Plot original FORCE/DIST data

;Check if we want clip this data
response=DIALOG_MESSAGE('Do you wish to clip this data',$
/QUESTION, /DEFAULT_NO, /CENTER, TITLE='Append File?')
if response EQ 'Yes' then begin
;select areas of plot to be cropped left mouse click on region to left and right mouse click on region to the right

CURSOR, X, Y, /DATA
WHILE (!MOUSE.button NE 4) DO BEGIN 
      CURSOR,  X1,  Y1, /DATA
      PLOTS,[X,X1], [Y,Y1], /DATA
   ENDWHILE 
PRINT, "X = ", X  
PRINT, "X1 =", X1
PRINT, "Y =", Y
PRINT, "Y1 =", Y1

index = WHERE(Dist_trav, count)
Print, "Index for Distance traveled =",Index
result = Dist_trav[Index]
;PRINT, "Distance Data Values =",Result  ;Actual Data Values

;  find values to discard
Lower_Values_to_Discard = WHERE(Result LT X )
PRINT,"Lower Values to Discard =",Lower_Values_to_Discard
Higher_Values_to_Discard = WHERE(Result GT X1)
PRINT,"Higher Values to Discard =",Higher_Values_to_Discard

;  Extract Maximum lower index value
Max_Lower=MIN(Lower_Values_to_Discard)
Print,"MAXIMUM LOWER Index= ",Max_Lower

;  Extract Minimum Higher index value
Min_Higher=MAX(Higher_Values_to_Discard)
Print,"MINIMUMHIGHER=",Min_Higher

;Create new Clipped dist_traveled and Res_Force arrays
;
ClippedX=Dist_trav[Min_Higher:Max_Lower]
ClippedY=Res_Force[Min_Higher:Max_Lower]

;  plot clipped data
WINDOW,27,TITLE='Clipped Force\Distance 10 Degree Insertion  Tip 94',XSIZE=800,YSIZE=720 ;put name of experiment in title i.e tip 92
plot,ClippedX,ClippedY,TITLE='Tip 94 10 Insert Force/Distance ',XTITLE='Metres',YTITLE='Newtons',THICK=1.25,POS=[0.125,0.125,0.95,0.95],CHARSIZE=1.25,CHARTHICK=1.0,LINESTYLE=0; Plot Clipped FORCE/DIST data

;Save Clipped Data
X=ClippedX
Y=ClippedY
YMax = Max(Y)
print, "Max y = ", YMax
filters = ['*.dat'] 
filename1 = DIALOG_PICKFILE(/WRITE,FILTER=filters)
print,filename1

;  Write new plot data to file for overplotting later
OPENW,1,filename1
WRITEU,1,X,Y
CLOSE,1
print,'finished'

endif
YMax = Max(Y)
print, "Max y = ",YMax 
;Check if we want save this data
savedata=DIALOG_MESSAGE('Do you wish to save this data as it is',$
/QUESTION, /DEFAULT_NO, /CENTER, TITLE='Append File?')

if savedata EQ 'Yes' then begin  
  ;Save Original Data
  X=Dist_trav
  Y=Res_Force

  filters = ['*.dat'] 
  filename1 = DIALOG_PICKFILE(/WRITE,FILTER=filters)

  print,filename1;Write original data to file for overplotting later
  OPENW,1,filename1
  WRITEU,1,X,Y
  CLOSE,1
  print,'finished'
endif
END