Pro optical_trap_force1

;Set up constants and arrays. NUMFRAMES is the total number of time steps in the data
NUMFRAMES=37L
Frame_Index=LON64ARR(NUMFRAMES)
Time_Index=FLTARR(NUMFRAMES)
X_Pix=INTARR(NUMFRAMES)
Y_Pix=INTARR(NUMFRAMES)
X_um=FLTARR(NUMFRAMES)
Y_um=FLTARR(NUMFRAMES)
window_y_index=0L
window_x_index=0L
window_vert_inc=25L

;Select and read an ASCII file containing the data
FILE=DIALOG_PICKFILE()
Template=ASCII_TEMPLATE(FILE, BROWSE_LINES=NUMFRAMES)
Buffer=READ_ASCII(FILE,DATA_START=2,TEMPLATE=Template)

;Convert the structure "Buffer" into component arrays
Frame_Index=Buffer.Field1
Time_Index=Buffer.Field2
X_Pix=Buffer.Field3
Y_Pix=Buffer.Field4
X_um=Buffer.Field5
Y_um=Buffer.Field6

;Rewrite Time_Index as 1/500 sec  High speed camera frame rate
Time_Int_5=1./500       ;High speed camera 500fps
Time_Int_K=1./1000      ;High speed camera 1000fps
Time_Int_C = 1./55      ;Conventional; Camera 55 fps
Time_Index[*]=0.0
Time=0.0

;Ensure that the correct Time_Int is selected for the relevant framerate
for i=0,NUMFRAMES-1 do begin
  Time_Index[i]=Time
  Time=Time+Time_Int_K  ;500 fps
endfor
;print, "TI =",Time_Index
;Print,"TIME=",Time
;print, Frame_Index

;Calculate the derivatives culled, X & Y time histories (Velocity)
X_Deriv=FLTARR(NUMFRAMES)
Y_Deriv=FLTARR(NUMFRAMES)
X_Deriv=DERIV(Time_Index,X_um)
Y_Deriv=DERIV(Time_Index,Y_um)

;;Smooth the  derivatives to get the final answer.
X_Deriv_S=SMOOTH(X_Deriv,21)  ;smooth the data
Y_Deriv_S=SMOOTH(Y_Deriv,21)  ;smooth the data
;X_Deriv_S=X_Deriv    ;remove smoothing
;Y_Deriv_S=Y_Deriv    ;remove smoothing

;Print,"X_Deriv_S = ",X_Deriv_S
;Print,"Y_Deriv_S = ",Y_Deriv_S

;Apply a cubic spline fit to the data
;Fit a cubic spline to the culled data
X_track_Int=FLTARR(NUMFRAMES)
Y_track_Int=FLTARR(NUMFRAMES)
X_track_Int=SPL_INIT(Time_Index,X_Deriv_S)
Y_track_Int=SPL_INIT(Time_Index,Y_Deriv_S)

;Calculate a regular set of NUMFRAMES Time points over the interval
End_Time=Float(NUMFRAMES+1) 
Time_Interval=(End_Time-Time)/FLOAT(NUMFRAMES+1)
Interp_Time=FLTARR(NUMFRAMES)
Time=0.0
for i=0,NUMFRAMES-1 do begin
  Interp_Time[i]=Time
  Time=Time+Time_Interval
endfor

;Use Cubic Splines to interpolate and smooth the X and Y data at the Interp_Time points
Interp_X=SPL_INTERP(Time_Index,X_Deriv_S,X_track_Int,Interp_Time)
Interp_Y=SPL_INTERP(Time_Index,Y_Deriv_S,Y_track_Int,Interp_Time)


;Calculate the acceleration for the Inertia force (F = ma)
X_Deriv2 = FLTARR(NUMFRAMES)
Y_Deriv2 = FLTARR(NUMFRAMES)
X_Deriv2=Deriv(Time_Index,X_Deriv_S)
Y_Deriv2=Deriv(Time_Index,Y_Deriv_S)
; Density of silical dioxide =2.65g/cm^3     Only for glass sphere insert alternative material densities if required
Sio2 = 2.65                                 ;Density of silica glass (g/cm^3)
PI=3.14159                                  ;PI
MU=1.002e-03                                ;Viscocity of medium (water)              
r=1.5e-6                                    ;Radius of particle
Vol = ((4./3)*(PI*r^3))                     ;Volume of a sphere
Mass = Vol*Sio2                             ;Mass = Vol*Density
Print,"mass = ",Mass                                          ;Take into account the boundary conditions of the microscope slide  
b = 1.5e-6                                  ;b is the distance to the nearest boundary      
a = (1/(1-((9./16)*(r/b))+((1./8)*(r/b)^3)-((45./256)*(r/b)^4)-((1./16)*(r/b)^5)))      ;Boundary Coefficient

;Inertia Force
;Calculate the x and y components of teh force
FI_X = FLTARR(NUMFRAMES)
FI_Y = FLTARR(NUMFRAMES)
for i=0,NUMFRAMES-1 do begin 
 FI_X[i]=Mass*X_Deriv2[i]*1.0e-06        ; X_Deriv(velocity) is in um/sec
 FI_X[i]=Mass*X_Deriv2[i]*1.0e-06
 endfor

;Stokes Drag Force
;Calculate the x and y component of the force
Force_x=FLTARR(NUMFRAMES)
Force_y=FLTARR(NUMFRAMES)
Total_Force = FLTARR(NUMFRAMES)
Force_Constant=6.0*PI*MU*r*a              ;Stokes Drag force coefficient * boundary coefficient (a)
for i=0,NUMFRAMES-1 do begin
 Force_x[i]=Force_Constant*Interp_X[i]*1.0e-06        ; X_Deriv(velocity) is in um/sec
 Force_y[i]=Force_Constant*Interp_X[i]*1.0e-06
 endfor

;The Inertia force can be neglected if the reynolds number (RE)<<1 
;RE = p*v*(D/MU) Where p is the density of the medium, v is the max velocity
; & D is the particle diameter, n = visosity of the medium
D = r^2
p = 999.97                        ;Density of water at 20 degrees C (Kg/m^3)
for i = 0,NUMFRAMES-1 do begin
VXmax = Max(X_Deriv_S[i])*1.e-6
VYmax = Max(Y_Deriv_S[i])*1.e-6
endfor
VTMax = sqrt(VXmax^2 +VYmax^2) ;Overall max velocity
RE = p*VTmax*(D/MU)
Print,"RE = ",RE
; Inertia force only included if RE > 1
If (RE lt 1) then for i=0,NUMFRAMES-1 do begin         ;Total force minus Inertia force
 Force_x[i]=Force_Constant*X_Deriv_S[i]*1.0e-06        ; X_Deriv(velocity) is in um/sec
 Force_y[i]=Force_Constant*Y_Deriv_S[i]*1.0e-06
endfor else begin 
if (RE GT 1)then for i=0,NUMFRAMES-1 do begin              ;Total Force + Inertia force
 Force_x[i]=Force_Constant*X_Deriv_S[i]+FI_X[i]*1.0e-06        ; FI_X = inertia force 
 Force_y[i]=Force_Constant*Y_Deriv_S[i]+FI_Y[i]*1.0e-06        
 endfor
    endelse   
                        
;Minimum and Maximum force values 
FORX = FLTARR(NUMFRAMES)
FORY = FLTARR(NUMFRAMES)
Tot_F = FLTARR(NUMFRAMES)
for i=0, NUMFRAMES-1 do begin
  FORX[i] = sqrt(Force_x[i]^2.0)           ;Make force positive to allow correct max and min values
  FORY[i] = sqrt(Force_y[i]^2.0)           ;Make force positive to allow correct max and min values
  Tot_F[i] = sqrt(FORX[i]^2.0+FORY[i]^2)   ;Total XY Maximum force 
  endfor
  
Min_Force =MIN(Tot_F)     
Max_Force =MAX(Tot_F)
print, "Minimum Force = ", Min_Force , "Maximum Force = ", Max_Force

;Check if we want to append this line to the combined data file
response=DIALOG_MESSAGE('Do you wish to append this data to the current combined data file?',$
/QUESTION, /DEFAULT_NO, /CENTER, TITLE='Append File?')
if response EQ 'Yes' then begin
  print, 'Adding data to combined file...'
  ;Does the file exist?
  exists=FILE_TEST('C:\Users\Steven Ross\Desktop\XYDataForceResults\Tip 94 10 Insert 500a\Optical_trap_force\XYForce.dat')
  if exists then begin
    print,'The file exists - appending data...'
    ;read in the existing data
    OPENU, Unit, 'C:\Users\Steven Ross\Desktop\XYDataForceResults\Tip 94 10 Insert 500a\Optical_trap_force\XYForce.dat', /GET_LUN, /NOEXPAND_PATH
    NUMFRAMES_PREV=3000L      ; We need to give this a value to initiate the variable
    READU, Unit, NUMFRAMES_PREV
    print,"NUMFRAMES_PREV in file=",NUMFRAMES_PREV
    X_um_prev=FLTARR(NUMFRAMES_PREV)
    Y_um_prev=FLTARR(NUMFRAMES_PREV)
    Force_X_prev=FLTARR(NUMFRAMES_PREV)
    Force_Y_prev=FLTARR(NUMFRAMES_PREV)
    READU, Unit, X_um_prev,Y_um_prev,Force_x_prev,Force_y_prev
    CLOSE, Unit
    FREE_LUN, Unit
    ;add on the new data
    NUMFRAMES_NEW=NUMFRAMES_PREV+NUMFRAMES
    X_um_new=FLTARR(NUMFRAMES_NEW)
    Y_um_new=FLTARR(NUMFRAMES_NEW)
    Force_X_new=FLTARR(NUMFRAMES_NEW)
    Force_Y_new=FLTARR(NUMFRAMES_NEW)
    X_um_new=[X_um_prev,X_um]
    Y_um_new=[Y_um_prev,Y_um]
    Force_X_new=[Force_X_prev,Force_x]
    Force_X_new=[Force_Y_prev,Force_y]
    ;delete the old file
    FILE_DELETE, 'C:\Users\Steven Ross\Desktop\XYDataForceResults\Tip 92 10 Insert 500\XYForce.dat', /NOEXPAND_PATH, /VERBOSE
    ;write the new expanded file
    print,'Writing the expanded file...'
    OPENW, Unit, 'C:\Users\Steven Ross\Desktop\XYDataForceResults\Tip 94 10 Insert 500a\Optical_trap_force\XYForce.dat', /GET_LUN
    WRITEU, Unit, NUMFRAMES_NEW
    WRITEU, Unit, X_um_new,Y_um_new,Force_X_new,Force_Y_new
    CLOSE, Unit
    FREE_LUN, Unit
  endif else begin
    print,'The file does not exist - creating file...'
    OPENW, Unit,'C:\Users\Steven Ross\Desktop\XYDataForceResults\Tip 94 10 Insert 500a\Optical_trap_force\XYForce.dat', /GET_LUN
    WRITEU, Unit, NUMFRAMES
    WRITEU, Unit, X_um,Y_um,Force_x,Force_y
    CLOSE, Unit
    FREE_LUN, Unit
  endelse
  
endif else begin
  print,"We said no"
endelse
print, "End"
End